# ---------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# ---------------------------------------------------------

"""Defines utilities for handling kwargs on SHAP-based explainers."""

from azureml.explain.model._internal.constants import ExplainParams


def _get_explain_global_kwargs(sampling_policy, method, include_local):
    """Get the kwargs for explain_global.

    :param sampling_policy: Optional policy for sampling the evaluation examples.  See documentation on
        SamplingPolicy for more information.
    :type sampling_policy: SamplingPolicy
    :param method: The explanation method used, e.g. shap_kernel, mimic
    :type method: str
    :param include_local: Whether a local explanation should be generated or only global
    :type include_local: bool
    :return: Args for explain_global.
    :rtype: dict
    """
    kwargs = {ExplainParams.METHOD: method,
              ExplainParams.SAMPLING_POLICY: sampling_policy,
              ExplainParams.INCLUDE_LOCAL: include_local}
    return kwargs
